#include "SBVTnode.h"

SBVTransp::SBVTransp() {};

void SBVTransp::setSBVTransp(double maxCapacitySBVT, double granularitySBVT)
{
    m_vecTransId.resize(1, 0);
    m_transSBVT = 0;
    m_maxCapacitySBVT = maxCapacitySBVT;
    m_granularitySBVT = granularitySBVT;
}

bool SBVTransp::emptyTransp()
{
    if (m_vecTransId.size() == 1 && m_vecTransId[0] == 0)
        return true;
    return false;
}

bool SBVTransp::allocatedSBVTranp(double rate)
{
    if (m_transSBVT == m_maxCapacitySBVT)
    {
        return false;
    }
    double tmpTrans = m_transSBVT + rate;
    if (m_maxCapacitySBVT >= tmpTrans)
    {
        return true;
    } else
        return false;
}

bool SBVTransp::existTransId(int id)
{
    for (int i = 0; i < m_vecTransId.size(); ++i)
    {
        if (m_vecTransId[i] == id)
            return true;
    }
    return false;
}

void SBVTransp::addTransId(double rate, int id)
{
    m_transSBVT = m_transSBVT + rate;
    if (emptyTransp() == true)
    {
        m_vecTransId[0] = id;
    } else {
        m_vecTransId.push_back(id);
    }

    if (m_transSBVT > m_maxCapacitySBVT)
    {
        cerr << "error with calculation of capacity SBVTransonder. SBVTransponder capacity is bigger than max "
                << endl;
        exit(1);
    }

}

void SBVTransp::removeTransId(double rate, int id)
{
    m_transSBVT = m_transSBVT - rate;
    if (m_transSBVT < 0)
    {
        cerr << "error with removeTransId of sbvt ujemne m_transSBVT " << endl;
        exit(1);
    }

    if (m_vecTransId.size() == 1)
    {
        m_vecTransId[0] = 0;

        if (m_transSBVT != 0)
        {
            cerr << "error with bit rate of SBVT. po removeTransId znajdujacego sie tylko jednego trans m_transSBVT!=0 "
                    << endl;
            exit(1);
        }
    }

    else
    {
        int idx;
        int i;
        for (i = 0; i < m_vecTransId.size(); ++i)
        {
            if (m_vecTransId[i] == id)

                break;
        }
        idx = i;
        //cout<< "sbvt przed remove: " ;
       //  showSBVTransp();

        m_vecTransId.erase(m_vecTransId.begin() + idx);
        // cout<< "sbvt po remove: " ;
        // showSBVTransp();
    }
}

bool SBVTransp::terminateTransId(double rate, int id)
{
    bool ret;
    if (existTransId(id) == true)
    {
        removeTransId(rate, id);
        ret = true;
    }
    else
    {
        cerr << "brak takiej trans id w sbvtransponder" << endl;
        exit(1);
        ret = false;
    }

    return ret;
}

double SBVTransp::get_transSBVTransp()
{
    return m_transSBVT;
}

void SBVTransp::showSBVTransp()
{
    for (int i = 0; i < m_vecTransId.size(); ++i)
    {
        cout << m_vecTransId[i] << " ";
    }
    cout << endl;
}

SBVTransp::~SBVTransp() {};
//---------------------------------------------------------------//
//int m_idxLink
SBVTnode::SBVTnode() {
}

void SBVTnode::initSBVTnode(int idxNode, double degree, double capacity,
        double granularity, int numberSBVTransp)
{
    m_idxNode = idxNode;
    m_maxCapacity = capacity;
    m_nSBVTransp = degree * numberSBVTransp;
    m_granularity = granularity;

    for (int i = 0; i < m_nSBVTransp; ++i)
    {
        SBVTransp sbvtransp;
        sbvtransp.setSBVTransp(capacity, granularity);
        m_vecSBVTransp.push_back(sbvtransp);
        //setMaxCapacity(degree, capacity);
    }
}
void SBVTnode:: setMaxCapacity( double degree, double capacity)
 {
 m_maxCapacity=degree*capacity;
 }

bool SBVTnode::enoughtCapacity(double bw)
{

    for (int i = 0; i < m_nSBVTransp; ++i)
    {
        if (m_vecSBVTransp[i].allocatedSBVTranp(bw) == true)
            return true;
    }
    return false;
}

void SBVTnode::addTransmission(double bw, int id)
{
    int idx;
       int i;
    for (i = 0; i < m_nSBVTransp; ++i)
    {
        if (m_vecSBVTransp[i].allocatedSBVTranp(bw) == true)
        {
            idx = i;
            break;
        }
    }
    //cout<< idx << endl;
    m_vecSBVTransp[idx].addTransId(bw, id);

}

void SBVTnode::dropTransmission(double bw, int id) {
    int idx;
    int i;
    for (i = 0; i < m_nSBVTransp; ++i) {
        if (m_vecSBVTransp[i].existTransId(id) == true) {
            m_vecSBVTransp[i].removeTransId(bw, id);
        }
    }
}


int SBVTnode::numSBVTransp()
{
    return m_nSBVTransp;
}

double SBVTnode::usedCapacity()
{
    double retCapacity;
    for (int i = 0; i < m_nSBVTransp; ++i)
    {
        retCapacity = retCapacity + m_vecSBVTransp[i].get_transSBVTransp();
    }
    return retCapacity;
}

double SBVTnode::ratioUtilizeSBVTnode() {
    double ret;
    ret = usedCapacity() / (m_nSBVTransp* m_maxCapacity);
    return ret;
}

void SBVTnode::showSBVT() {
    cout << " index wezla: " << m_idxNode;

    cout << " liczba transponderow w wezle " << m_nSBVTransp << " ";
    for (int i = 0; i < m_nSBVTransp; ++i) {
        m_vecSBVTransp[i].showSBVTransp();
    }
    cout << endl;
}

SBVTnode::~SBVTnode() {};

