#include "Net.h"

using namespace std;

vector<int> Net::loadLengthLinksFromFile(string filename, int nodes,
        int links) {

    vector<int> tmpLength;
    vector<vector<int> > topology;

    ifstream infile(filename.c_str());
    if (infile.is_open() == false) {
        cerr << "Error: Unable to read the specified network structure file."
                << endl;
        exit(1);
    }

    topology.resize(nodes);

    for (int i = 0; i < nodes; ++i) {
        topology[i].resize(nodes);

        for (int j = 0; j < nodes; ++j) {

            if (infile.eof() == true) {
                cerr << "Error: Insufficient number of coefficients." << endl;
                exit(1);
            }
            infile >> topology[i][j];
        }
    }
    infile.close();

    // Fill the vector of distance values
    for (int i = 0; i < nodes; ++i) {
        for (int j = 0; j < nodes; ++j) {
            if (topology[i][j] > 0) {
               // cout<< "dlugosc: "<<topology[i][j];
                tmpLength.push_back(topology[i][j]);
            }
        }
    }

    m_links = tmpLength;
    return m_links;
}

vector<vector<vector<PathSD> > > Net::setFromFile(string filename, int node,
        int link, int can) {

    ifstream infile;
    infile.open(filename.c_str(), ios::in);
    if (infile.is_open() == false) {
        cerr << "nie mozna otworzyc pliku" << endl;
        abort();
    }

    vector<vector<vector<PathSD> > > tmp1; //wektor wszystkich sciezek zawartych w pliku
    vector<PathSD> ptmp; //wektor pomocniczy
    bool val;
    ptmp.clear();
    vector<bool> vecp;
    vecp.clear();
    m_connection.resize(node);
    tmp1.resize(node);
    for (int i = 0; i < node; ++i) {
        m_connection[i].resize(node);
        tmp1[i].resize(node);
        for (int j = 0; j < node; ++j) {
            if (i != j) //jezeli src!=drst to utworz dla takiej pary wektor, ktory przechowuje sciezki
                    {
                tmp1[i][j].resize(can); // ten wektor wektorow <bool> przechowuje wszystkie sciezki dla danego zadania SD
                ptmp.resize(can);
                for (int k = 0; k < can; ++k) {

                    vecp.clear();
                    vecp.resize(link);
                    PathSD p1;
                    for (int l = 0; l < link; ++l) //liczba kolumn
                            {
                        infile >> val;
                        if (infile.eof() == true) {
                            cerr << "???";
                            exit(1);
                        }

                        vecp[l] = val;
                    }
                    p1.setPath(vecp);
                    p1.setLengthModul(m_links);
                    ptmp[k] = p1;
                }
                tmp1[i][j] = ptmp;

                ptmp.erase(ptmp.begin() + m_cand, ptmp.end());

                m_connection[i][j] = ptmp;

            }

        }


    }

    infile.close();
     return m_connection;
}


void Net::checkPaths()
{
    for(int i=0; i<m_nNodes; ++i)
                {
                for (int j = 0; j < m_nNodes; ++j)
                {
                    if(i!=j)

                    for (int k=0; k< m_cand; ++k)
                    {
                        if (m_connection[i][j][k].getModul()==-1)
                        {  cerr<< "two long path "<<endl;
                                abort();
                        }
                        else
                       ev<<"  modulacja sciezki i j cand:  " << i<< " " << j<< " "<< k << " "<< m_connection[i][j][k].getModul()<<endl;



                    }
                }
                }
                }


vector<vector<CyclePath> > Net::setCyclic(int nodes, int cand) {
    m_currIdx.resize(nodes);
    for (int i = 0; i < nodes; ++i) {
        m_currIdx[i].resize(nodes);
        for (int j = 0; j < nodes; ++j) {
            if (i != j) {

                m_currIdx[i][j].initCycle(cand);
            }
        }
    }
    return m_currIdx;
}

void Net::setCand(int can) {
    m_cand = can;
}
int Net::numCand() {
    return m_cand;
}

vector<PathSD> Net::candPaths(int src, int dest) {
    vector<PathSD> ret;

    for (int i = 0; i < m_cand; ++i)

    {
        ret.push_back(m_connection[src][dest][i]);
    }

    return ret;
}
//----------------------------------------------//
int Net::calcHop(vector<bool> pathAlloc) {
    int nhop = 0;
    for (int i = 0; i < pathAlloc.size(); ++i) {
        if (pathAlloc[i] != 0) {
            nhop++;
        }
    }
    return nhop;
}
int Net::calcLen(vector<bool> pathAlloc) {

    int len = 0;
    for (int i = 0; i < pathAlloc.size(); ++i) {
        len = len + pathAlloc[i] * m_links[i];
    }
    return len;
}

//-------------------------------------------------------------------------//
void Net::showVector(vector<int> myVec) {
    for (int i = 0; i < myVec.size(); ++i) {
        ev<<"nr linku: " <<i << " dl linku: " << myVec[i]<<endl;

    }
}

//--------------------------//
double Net::numberBlockedBypass()
{
    return numBlockedBypass;
}

void Net::set_blockedBypassParameters(double val)
{
    numBlockedBypass=val;
bwBlockedBypass=val;
}

double Net:: getBwBlockedBypass()
{
    return bwBlockedBypass;

}

